@@ -14,7 +14,6 @@ import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
 
 // import org.checkerframework.checker.index.qual.Positive;
-// import org.checkerframework.checker.nullness.qual.NonNull;
 
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
@@ -50,7 +49,7 @@ import java.util.Map;
  */
 final class ArrayEncoding {
 
-  public interface ArrayEncoder<A> {
+  interface ArrayEncoder<A> {
 
     /**
      * The default array type oid supported by this instance.
@@ -102,6 +101,19 @@ final class ArrayEncoding {
      */
     byte[] toBinaryRepresentation(BaseConnection connection, A array, int oid)
         throws SQLException, SQLFeatureNotSupportedException;
+
+    /**
+     * Append {@code String} representation of <i>array</i> to <i>sb</i>.
+     *
+     * @param sb
+     *          The {@link StringBuilder} to append to.
+     * @param delim
+     *          The delimiter between elements.
+     * @param array
+     *          The array to represent. Will not be {@code null}, but may contain
+     *          {@code null} elements.
+     */
+    void appendArray(StringBuilder sb, char delim, A array);
   }
 
   /**
@@ -111,7 +123,7 @@ final class ArrayEncoding {
    * @param <A>
    *          Base array type supported.
    */
-  private abstract static class AbstractArrayEncoder<A extends /* @NonNull */ Object>
+  private abstract static class AbstractArrayEncoder<A extends Object>
       implements ArrayEncoder<A> {
 
     private final int oid;
@@ -193,19 +205,6 @@ final class ArrayEncoding {
       return sb.toString();
     }
 
-    /**
-     * Append {@code String} representation of <i>array</i> to <i>sb</i>.
-     *
-     * @param sb
-     *          The {@link StringBuilder} to append to.
-     * @param delim
-     *          The delimiter between elements.
-     * @param array
-     *          The array to represent. Will not be {@code null}, but may contain
-     *          {@code null} elements.
-     */
-    abstract void appendArray(StringBuilder sb, char delim, A array);
-
     /**
      * By default returns {@code true} if <i>oid</i> matches the <i>arrayOid</i>
      * this instance was instantiated with.
@@ -222,7 +221,7 @@ final class ArrayEncoding {
    * @param <N>
    *          The base type of array.
    */
-  private abstract static class NumberArrayEncoder<N extends Number> extends AbstractArrayEncoder<N /* @NonNull */ []> {
+  private abstract static class NumberArrayEncoder<N extends Number> extends AbstractArrayEncoder<N[]> {
 
     private final int fieldSize;
 
@@ -274,6 +273,8 @@ final class ArrayEncoding {
       ByteConverter.int4(bytes, 8, getTypeOID(oid));
       // length
       ByteConverter.int4(bytes, 12, array.length);
+      // postgresql uses 1 base by default
+      ByteConverter.int4(bytes, 16, 1);
 
       return bytes;
     }
@@ -327,7 +328,7 @@ final class ArrayEncoding {
      * {@inheritDoc}
      */
     @Override
-    final void appendArray(StringBuilder sb, char delim, N[] array) {
+    public final void appendArray(StringBuilder sb, char delim, N[] array) {
       sb.append('{');
       for (int i = 0; i < array.length; ++i) {
         if (i != 0) {
@@ -351,7 +352,7 @@ final class ArrayEncoding {
    * @param <A>
    *          The primitive array to support.
    */
-  private abstract static class FixedSizePrimitiveArrayEncoder<A extends /* @NonNull */ Object>
+  private abstract static class FixedSizePrimitiveArrayEncoder<A extends Object>
       extends AbstractArrayEncoder<A> {
 
     private final int fieldSize;
@@ -393,6 +394,8 @@ final class ArrayEncoding {
       ByteConverter.int4(bytes, 8, getTypeOID(oid));
       // length
       ByteConverter.int4(bytes, 12, arrayLength);
+      // postgresql uses 1 base by default
+      ByteConverter.int4(bytes, 16, 1);
 
       write(array, bytes, 20);
 
@@ -426,7 +429,7 @@ final class ArrayEncoding {
     protected abstract void write(A array, byte[] bytes, int offset);
   }
 
-  private static final AbstractArrayEncoder<long /* @NonNull */ []> LONG_ARRAY = new FixedSizePrimitiveArrayEncoder<long /* @NonNull */ []>(8, Oid.INT8,
+  private static final AbstractArrayEncoder<long[]> LONG_ARRAY = new FixedSizePrimitiveArrayEncoder<long[]>(8, Oid.INT8,
       Oid.INT8_ARRAY) {
 
     /**
@@ -458,7 +461,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Long /* @NonNull */ []> LONG_OBJ_ARRAY = new NumberArrayEncoder<Long>(8, Oid.INT8,
+  private static final AbstractArrayEncoder<Long[]> LONG_OBJ_ARRAY = new NumberArrayEncoder<Long>(8, Oid.INT8,
       Oid.INT8_ARRAY) {
 
     @Override
@@ -467,7 +470,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<int /* @NonNull */ []> INT_ARRAY = new FixedSizePrimitiveArrayEncoder<int /* @NonNull */ []>(4, Oid.INT4,
+  private static final AbstractArrayEncoder<int[]> INT_ARRAY = new FixedSizePrimitiveArrayEncoder<int[]>(4, Oid.INT4,
       Oid.INT4_ARRAY) {
 
     /**
@@ -499,7 +502,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Integer /* @NonNull */ []> INT_OBJ_ARRAY = new NumberArrayEncoder<Integer>(4, Oid.INT4,
+  private static final AbstractArrayEncoder<Integer[]> INT_OBJ_ARRAY = new NumberArrayEncoder<Integer>(4, Oid.INT4,
       Oid.INT4_ARRAY) {
 
     @Override
@@ -508,7 +511,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<short /* @NonNull */ []> SHORT_ARRAY = new FixedSizePrimitiveArrayEncoder<short /* @NonNull */ []>(2,
+  private static final AbstractArrayEncoder<short[]> SHORT_ARRAY = new FixedSizePrimitiveArrayEncoder<short[]>(2,
       Oid.INT2, Oid.INT2_ARRAY) {
 
     /**
@@ -540,7 +543,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Short /* @NonNull */ []> SHORT_OBJ_ARRAY = new NumberArrayEncoder<Short>(2, Oid.INT2,
+  private static final AbstractArrayEncoder<Short[]> SHORT_OBJ_ARRAY = new NumberArrayEncoder<Short>(2, Oid.INT2,
       Oid.INT2_ARRAY) {
 
     /**
@@ -552,7 +555,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<double /* @NonNull */ []> DOUBLE_ARRAY = new FixedSizePrimitiveArrayEncoder<double /* @NonNull */ []>(8,
+  private static final AbstractArrayEncoder<double[]> DOUBLE_ARRAY = new FixedSizePrimitiveArrayEncoder<double[]>(8,
       Oid.FLOAT8, Oid.FLOAT8_ARRAY) {
 
     /**
@@ -587,7 +590,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Double /* @NonNull */ []> DOUBLE_OBJ_ARRAY = new NumberArrayEncoder<Double>(8, Oid.FLOAT8,
+  private static final AbstractArrayEncoder<Double[]> DOUBLE_OBJ_ARRAY = new NumberArrayEncoder<Double>(8, Oid.FLOAT8,
       Oid.FLOAT8_ARRAY) {
 
     /**
@@ -599,7 +602,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<float /* @NonNull */ []> FLOAT_ARRAY = new FixedSizePrimitiveArrayEncoder<float /* @NonNull */ []>(4,
+  private static final AbstractArrayEncoder<float[]> FLOAT_ARRAY = new FixedSizePrimitiveArrayEncoder<float[]>(4,
       Oid.FLOAT4, Oid.FLOAT4_ARRAY) {
 
     /**
@@ -634,7 +637,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Float /* @NonNull */ []> FLOAT_OBJ_ARRAY = new NumberArrayEncoder<Float>(4, Oid.FLOAT4,
+  private static final AbstractArrayEncoder<Float[]> FLOAT_OBJ_ARRAY = new NumberArrayEncoder<Float>(4, Oid.FLOAT4,
       Oid.FLOAT4_ARRAY) {
 
     /**
@@ -646,7 +649,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<boolean /* @NonNull */ []> BOOLEAN_ARRAY = new FixedSizePrimitiveArrayEncoder<boolean /* @NonNull */ []>(1,
+  private static final AbstractArrayEncoder<boolean[]> BOOLEAN_ARRAY = new FixedSizePrimitiveArrayEncoder<boolean[]>(1,
       Oid.BOOL, Oid.BOOL_ARRAY) {
 
     /**
@@ -678,7 +681,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Boolean /* @NonNull */ []> BOOLEAN_OBJ_ARRAY = new AbstractArrayEncoder<Boolean /* @NonNull */ []>(Oid.BOOL,
+  private static final AbstractArrayEncoder<Boolean[]> BOOLEAN_OBJ_ARRAY = new AbstractArrayEncoder<Boolean[]>(Oid.BOOL,
       Oid.BOOL_ARRAY) {
 
     /**
@@ -701,6 +704,8 @@ final class ArrayEncoding {
       ByteConverter.int4(bytes, 8, getTypeOID(oid));
       // length
       ByteConverter.int4(bytes, 12, array.length);
+      // postgresql uses 1 base by default
+      ByteConverter.int4(bytes, 16, 1);
 
       return bytes;
     }
@@ -743,7 +748,7 @@ final class ArrayEncoding {
      * {@inheritDoc}
      */
     @Override
-    void appendArray(StringBuilder sb, char delim, Boolean[] array) {
+    public void appendArray(StringBuilder sb, char delim, Boolean[] array) {
       sb.append('{');
       for (int i = 0; i < array.length; ++i) {
         if (i != 0) {
@@ -759,7 +764,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<String /* @NonNull */ []> STRING_ARRAY = new AbstractArrayEncoder<String /* @NonNull */ []>(Oid.VARCHAR,
+  private static final AbstractArrayEncoder<String[]> STRING_ARRAY = new AbstractArrayEncoder<String[]>(Oid.VARCHAR,
       Oid.VARCHAR_ARRAY) {
 
     /**
@@ -846,8 +851,8 @@ final class ArrayEncoding {
         ByteConverter.int4(buffer, 0, array.length);
         baos.write(buffer);
 
-        // write 4 empty bytes
-        java.util.Arrays.fill(buffer, (byte) 0);
+        // postgresql uses 1 base by default
+        ByteConverter.int4(buffer, 0, 1);
         baos.write(buffer);
 
         final Encoding encoding = connection.getEncoding();
@@ -916,7 +921,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<byte[] /* @NonNull */ []> BYTEA_ARRAY = new AbstractArrayEncoder<byte[] /* @NonNull */ []>(Oid.BYTEA,
+  private static final AbstractArrayEncoder<byte[][]> BYTEA_ARRAY = new AbstractArrayEncoder<byte[][]>(Oid.BYTEA,
       Oid.BYTEA_ARRAY) {
 
     /**
@@ -951,6 +956,8 @@ final class ArrayEncoding {
       ByteConverter.int4(bytes, 8, getTypeOID(oid));
       // length
       ByteConverter.int4(bytes, 12, array.length);
+      // postgresql uses 1 base by default
+      ByteConverter.int4(bytes, 16, 1);
 
       write(array, bytes, 20);
 
@@ -1009,7 +1016,7 @@ final class ArrayEncoding {
      * {@inheritDoc}
      */
     @Override
-    void appendArray(StringBuilder sb, char delim, byte[][] array) {
+    public void appendArray(StringBuilder sb, char delim, byte[][] array) {
       sb.append('{');
       for (int i = 0; i < array.length; ++i) {
         if (i > 0) {
@@ -1035,7 +1042,7 @@ final class ArrayEncoding {
     }
   };
 
-  private static final AbstractArrayEncoder<Object /* @NonNull */ []> OBJECT_ARRAY = new AbstractArrayEncoder<Object /* @NonNull */ []>(0, 0) {
+  private static final AbstractArrayEncoder<Object[]> OBJECT_ARRAY = new AbstractArrayEncoder<Object[]>(0, 0) {
 
     @Override
     public int getDefaultArrayTypeOid() {
@@ -1063,7 +1070,7 @@ final class ArrayEncoding {
     }
 
     @Override
-    void appendArray(StringBuilder sb, char delim, Object[] array) {
+    public void appendArray(StringBuilder sb, char delim, Object[] array) {
       sb.append('{');
       for (int i = 0; i < array.length; ++i) {
         if (i > 0) {
@@ -1071,6 +1078,16 @@ final class ArrayEncoding {
         }
         if (array[i] == null) {
           sb.append('N').append('U').append('L').append('L');
+        } else if (array[i].getClass().isArray()) {
+          if (array[i] instanceof byte[]) {
+            throw new UnsupportedOperationException("byte[] nested inside Object[]");
+          }
+          try {
+            getArrayEncoder(array[i]).appendArray(sb, delim, array[i]);
+          } catch (PSQLException e) {
+            // this should never happen
+            throw new IllegalStateException(e);
+          }
         } else {
           PgArray.escapeArrayElement(sb, array[i].toString());
         }
@@ -1080,7 +1097,7 @@ final class ArrayEncoding {
   };
 
   @SuppressWarnings("rawtypes")
-  private static final Map</* @NonNull */ Class, /* @NonNull */ AbstractArrayEncoder> ARRAY_CLASS_TO_ENCODER = new HashMap</* @NonNull */ Class, /* @NonNull */ AbstractArrayEncoder>(
+  private static final Map<Class, AbstractArrayEncoder> ARRAY_CLASS_TO_ENCODER = new HashMap<Class, AbstractArrayEncoder>(
       (int) (14 / .75) + 1);
 
   static {
@@ -1112,7 +1129,7 @@ final class ArrayEncoding {
    * @see ArrayEncoding.ArrayEncoder#supportBinaryRepresentation(int)
    */
   @SuppressWarnings({ "unchecked", "rawtypes" })
-  public static <A extends /* @NonNull */ Object> ArrayEncoder<A> getArrayEncoder(A array) throws PSQLException {
+  public static <A extends Object> ArrayEncoder<A> getArrayEncoder(A array) throws PSQLException {
     final Class<?> arrayClazz = array.getClass();
     Class<?> subClazz = arrayClazz.getComponentType();
     if (subClazz == null) {
@@ -1160,7 +1177,7 @@ final class ArrayEncoding {
    * Wraps an {@link AbstractArrayEncoder} implementation and provides optimized
    * support for 2 dimensions.
    */
-  private static final class TwoDimensionPrimitiveArrayEncoder<A extends /* @NonNull */ Object> implements ArrayEncoder<A /* @NonNull */ []> {
+  private static final class TwoDimensionPrimitiveArrayEncoder<A extends Object> implements ArrayEncoder<A[]> {
     private final AbstractArrayEncoder<A> support;
 
     /**
@@ -1184,8 +1201,17 @@ final class ArrayEncoding {
      * {@inheritDoc}
      */
     @Override
-    public String toArrayString(char delim, A /* @NonNull */[] array) {
+    public String toArrayString(char delim, A[] array) {
       final StringBuilder sb = new StringBuilder(1024);
+      appendArray(sb, delim, array);
+      return sb.toString();
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void appendArray(StringBuilder sb, char delim, A[] array) {
       sb.append('{');
       for (int i = 0; i < array.length; ++i) {
         if (i > 0) {
@@ -1194,7 +1220,6 @@ final class ArrayEncoding {
         support.appendArray(sb, delim, array[i]);
       }
       sb.append('}');
-      return sb.toString();
     }
 
     /**
@@ -1236,14 +1261,14 @@ final class ArrayEncoding {
         // length
         ByteConverter.int4(buffer, 0, array.length);
         baos.write(buffer);
-        // write 4 empty bytes
-        java.util.Arrays.fill(buffer, (byte) 0);
+        // postgres defaults to 1 based lower bound
+        ByteConverter.int4(buffer, 0, 1);
         baos.write(buffer);
 
         ByteConverter.int4(buffer, 0, array.length > 0 ? Array.getLength(array[0]) : 0);
         baos.write(buffer);
-        // write 4 empty bytes
-        java.util.Arrays.fill(buffer, (byte) 0);
+        // postgresql uses 1 base by default
+        ByteConverter.int4(buffer, 0, 1);
         baos.write(buffer);
 
         for (int i = 0; i < array.length; ++i) {
@@ -1299,6 +1324,14 @@ final class ArrayEncoding {
       return sb.toString();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void appendArray(StringBuilder sb, char delim, Object array) {
+      arrayString(sb, array, delim, dimensions);
+    }
+
     private void arrayString(StringBuilder sb, Object array, char delim, int depth) {
 
       if (depth > 1) {
@@ -1362,8 +1395,8 @@ final class ArrayEncoding {
         // length
         ByteConverter.int4(buffer, 0, Array.getLength(array));
         baos.write(buffer);
-        // write 4 empty bytes for lower bounds value. this is
-        java.util.Arrays.fill(buffer, (byte) 0);
+        // postgresql uses 1 base by default
+        ByteConverter.int4(buffer, 0, 1);
         baos.write(buffer);
 
         writeArray(connection, buffer, baos, array, dimensions, true);
@@ -1384,8 +1417,8 @@ final class ArrayEncoding {
       if (first) {
         ByteConverter.int4(buffer, 0, length > 0 ? Array.getLength(Array.get(array, 0)) : 0);
         baos.write(buffer);
-        // write 4 empty bytes
-        java.util.Arrays.fill(buffer, (byte) 0);
+        // postgresql uses 1 base by default
+        ByteConverter.int4(buffer, 0, 1);
         baos.write(buffer);
       }
 
