@@ -71,6 +71,7 @@ import java.sql.Statement;
 import java.sql.Struct;
 import java.sql.Types;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -160,6 +161,11 @@ public class PgConnection implements BaseConnection {
   // Default forcebinary option.
   protected boolean forcebinary = false;
 
+  /**
+   * Oids for which binary transfer should be disabled.
+   */
+  private final Set<? extends Integer> binaryDisabledOids;
+
   private int rsHoldability = ResultSet.CLOSE_CURSORS_AT_COMMIT;
   private int savepointId = 0;
   // Connection's autocommit state.
@@ -167,7 +173,7 @@ public class PgConnection implements BaseConnection {
   // Connection's readonly state.
   private boolean readOnly = false;
   // Filter out database objects for which the current user has no privileges granted from the DatabaseMetaData
-  private boolean  hideUnprivilegedObjects ;
+  private final boolean  hideUnprivilegedObjects ;
   // Whether to include error details in logging and exceptions
   private final boolean logServerErrorDetail;
   // Bind String to UNSPECIFIED or VARCHAR?
@@ -234,7 +240,7 @@ public class PgConnection implements BaseConnection {
 
     this.creatingURL = url;
 
-    this.readOnlyBehavior = getReadOnlyBehavior(PGProperty.READ_ONLY_MODE.get(info));
+    this.readOnlyBehavior = getReadOnlyBehavior(PGProperty.READ_ONLY_MODE.getOrDefault(info));
 
     setDefaultFetchSize(PGProperty.DEFAULT_ROW_FETCH_SIZE.getInt(info));
 
@@ -261,7 +267,14 @@ public class PgConnection implements BaseConnection {
 
     this.hideUnprivilegedObjects = PGProperty.HIDE_UNPRIVILEGED_OBJECTS.getBoolean(info);
 
-    Set<Integer> binaryOids = getBinaryOids(info);
+    // get oids that support binary transfer
+    Set<Integer> binaryOids = getBinaryEnabledOids(info);
+    // get oids that should be disabled from transfer
+    binaryDisabledOids = getBinaryDisabledOids(info);
+    // if there are any, remove them from the enabled ones
+    if (!binaryDisabledOids.isEmpty()) {
+      binaryOids.removeAll(binaryDisabledOids);
+    }
 
     // split for receive and send for better control
     Set<Integer> useBinarySendForOids = new HashSet<Integer>(binaryOids);
@@ -287,7 +300,7 @@ public class PgConnection implements BaseConnection {
     // String -> text or unknown?
     //
 
-    String stringType = PGProperty.STRING_TYPE.get(info);
+    String stringType = PGProperty.STRING_TYPE.getOrDefault(info);
     if (stringType != null) {
       if (stringType.equalsIgnoreCase("unspecified")) {
         bindStringAsVarchar = false;
@@ -331,7 +344,7 @@ public class PgConnection implements BaseConnection {
 
     this.clientInfo = new Properties();
     if (haveMinimumServerVersion(ServerVersion.v9_0)) {
-      String appName = PGProperty.APPLICATION_NAME.get(info);
+      String appName = PGProperty.APPLICATION_NAME.getOrDefault(info);
       if (appName == null) {
         appName = "";
       }
@@ -343,9 +356,9 @@ public class PgConnection implements BaseConnection {
             Math.max(0, PGProperty.DATABASE_METADATA_CACHE_FIELDS_MIB.getInt(info) * 1024L * 1024L),
         false);
 
-    replicationConnection = PGProperty.REPLICATION.get(info) != null;
+    replicationConnection = PGProperty.REPLICATION.getOrDefault(info) != null;
 
-    xmlFactoryFactoryClass = PGProperty.XML_FACTORY_FACTORY.get(info);
+    xmlFactoryFactoryClass = PGProperty.XML_FACTORY_FACTORY.getOrDefault(info);
   }
 
   private static ReadOnlyBehavior getReadOnlyBehavior(String property) {
@@ -388,28 +401,51 @@ public class PgConnection implements BaseConnection {
         Oid.UUID));
   }
 
-  private static Set<Integer> getBinaryOids(Properties info) throws PSQLException {
+  /**
+   * Gets all oids for which binary transfer can be enabled.
+   *
+   * @param info properties
+   * @return oids for which binary transfer can be enabled
+   * @throws PSQLException if any oid is not valid
+   */
+  private static Set<Integer> getBinaryEnabledOids(Properties info) throws PSQLException {
+    // check if binary transfer should be enabled for built-in types
     boolean binaryTransfer = PGProperty.BINARY_TRANSFER.getBoolean(info);
-    // Formats that currently have binary protocol support
+    // get formats that currently have binary protocol support
     Set<Integer> binaryOids = new HashSet<Integer>(32);
     if (binaryTransfer) {
       binaryOids.addAll(SUPPORTED_BINARY_OIDS);
     }
-
-    String oids = PGProperty.BINARY_TRANSFER_ENABLE.get(info);
+    // add all oids which are enabled for binary transfer by the creator of the connection
+    String oids = PGProperty.BINARY_TRANSFER_ENABLE.getOrDefault(info);
     if (oids != null) {
       binaryOids.addAll(getOidSet(oids));
     }
-    oids = PGProperty.BINARY_TRANSFER_DISABLE.get(info);
-    if (oids != null) {
-      binaryOids.removeAll(getOidSet(oids));
-    }
-
     return binaryOids;
   }
 
-  private static Set<Integer> getOidSet(String oidList) throws PSQLException {
-    Set<Integer> oids = new HashSet<Integer>();
+  /**
+   * Gets all oids for which binary transfer should be disabled.
+   *
+   * @param info properties
+   * @return oids for which binary transfer should be disabled
+   * @throws PSQLException if any oid is not valid
+   */
+  private static Set<? extends Integer> getBinaryDisabledOids(Properties info)
+      throws PSQLException {
+    // check for oids that should explicitly be disabled
+    String oids = PGProperty.BINARY_TRANSFER_DISABLE.getOrDefault(info);
+    if (oids == null) {
+      return Collections.emptySet();
+    }
+    return getOidSet(oids);
+  }
+
+  private static Set<? extends Integer> getOidSet(String oidList) throws PSQLException {
+    if (oidList.isEmpty()) {
+      return Collections.emptySet();
+    }
+    Set<Integer> oids = new HashSet<>();
     StringTokenizer tokenizer = new StringTokenizer(oidList, ",");
     while (tokenizer.hasMoreTokens()) {
       String oid = tokenizer.nextToken();
@@ -434,6 +470,7 @@ public class PgConnection implements BaseConnection {
 
   private final TimestampUtils timestampUtils;
 
+  @Deprecated
   public TimestampUtils getTimestampUtils() {
     return timestampUtils;
   }
@@ -706,14 +743,26 @@ public class PgConnection implements BaseConnection {
     try {
       addDataType(type, Class.forName(name).asSubclass(PGobject.class));
     } catch (Exception e) {
-      throw new RuntimeException("Cannot register new type: " + e);
+      throw new RuntimeException("Cannot register new type " + type, e);
     }
   }
 
   @Override
   public void addDataType(String type, Class<? extends PGobject> klass) throws SQLException {
     checkClosed();
+    // first add the data type to the type cache
     typeCache.addDataType(type, klass);
+    // then check if this type supports binary transfer
+    if (PGBinaryObject.class.isAssignableFrom(klass) && getPreferQueryMode() != PreferQueryMode.SIMPLE) {
+      // try to get an oid for this type (will return 0 if the type does not exist in the database)
+      int oid = typeCache.getPGType(type);
+      // check if oid is there and if it is not disabled for binary transfer
+      if (oid > 0 && !binaryDisabledOids.contains(oid)) {
+        // allow using binary transfer for receiving and sending of this type
+        queryExecutor.addBinaryReceiveOid(oid);
+        queryExecutor.addBinarySendOid(oid);
+      }
+    }
   }
 
   // This initialises the objectTypes hash map
