@@ -54,7 +54,7 @@ public class LibPQFactory extends WrappedFactory {
       Properties info) throws PSQLException {
     // Determine the callback handler
     CallbackHandler cbh;
-    String sslpasswordcallback = PGProperty.SSL_PASSWORD_CALLBACK.get(info);
+    String sslpasswordcallback = PGProperty.SSL_PASSWORD_CALLBACK.getOrDefault(info);
     if (sslpasswordcallback != null) {
       try {
         cbh = ObjectFactory.instantiate(CallbackHandler.class, sslpasswordcallback, info, false, null);
@@ -65,7 +65,7 @@ public class LibPQFactory extends WrappedFactory {
           PSQLState.CONNECTION_FAILURE, e);
       }
     } else {
-      cbh = new ConsoleCallbackHandler(PGProperty.SSL_PASSWORD.get(info));
+      cbh = new ConsoleCallbackHandler(PGProperty.SSL_PASSWORD.getOrDefault(info));
     }
     return cbh;
   }
@@ -75,7 +75,7 @@ public class LibPQFactory extends WrappedFactory {
       String sslkeyfile, String defaultdir, Properties info) throws  PSQLException {
 
     // Load the client's certificate and key
-    String sslcertfile = PGProperty.SSL_CERT.get(info);
+    String sslcertfile = PGProperty.SSL_CERT.getOrDefault(info);
     if (sslcertfile == null) { // Fall back to default
       defaultfile = true;
       sslcertfile = defaultdir + "postgresql.crt";
@@ -111,7 +111,7 @@ public class LibPQFactory extends WrappedFactory {
         defaultdir = System.getProperty("user.home") + pathsep + ".postgresql" + pathsep;
       }
 
-      String sslkeyfile = PGProperty.SSL_KEY.get(info);
+      String sslkeyfile = PGProperty.SSL_KEY.getOrDefault(info);
       if (sslkeyfile == null) { // Fall back to default
         defaultfile = true;
         sslkeyfile = defaultdir + "postgresql.pk8";
@@ -139,7 +139,7 @@ public class LibPQFactory extends WrappedFactory {
           // this should never happen
           throw new NoSuchAlgorithmException("jks KeyStore not available");
         }
-        String sslrootcertfile = PGProperty.SSL_ROOT_CERT.get(info);
+        String sslrootcertfile = PGProperty.SSL_ROOT_CERT.getOrDefault(info);
         if (sslrootcertfile == null) { // Fall back to default
           sslrootcertfile = defaultdir + "root.crt";
         }
