@@ -9,6 +9,7 @@ import static java.util.regex.Pattern.compile;
 
 // import org.checkerframework.checker.nullness.qual.Nullable;
 
+import java.util.Locale;
 import java.util.Objects;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -85,7 +86,7 @@ public class HostSpec {
     String separator = "";
     for (String disjunct : mask.split("\\|")) {
       if (!disjunct.isEmpty()) {
-        String regex = disjunctToRegex(disjunct.toLowerCase());
+        String regex = disjunctToRegex(disjunct.toLowerCase(Locale.ROOT));
         joiner.append(separator).append(regex);
         separator = "|";
       }
