@@ -10,6 +10,7 @@ import static org.junit.Assert.fail;
 
 import org.postgresql.PGConnection;
 import org.postgresql.core.BaseConnection;
+import org.postgresql.core.Oid;
 import org.postgresql.geometric.PGbox;
 import org.postgresql.geometric.PGpoint;
 import org.postgresql.jdbc.PgArray;
@@ -87,10 +88,11 @@ public class ArrayTest extends BaseTest4 {
 
   @Test
   public void testSetPrimitiveObjects() throws SQLException {
+    final String stringWithNonAsciiWhiteSpace = "a\u2001b";
     PreparedStatement pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
-    pstmt.setObject(1, new int[]{1,2,3}, Types.ARRAY);
+    pstmt.setObject(1, new int[]{1, 2, 3}, Types.ARRAY);
     pstmt.setObject(2, new double[]{3.1d, 1.4d}, Types.ARRAY);
-    pstmt.setObject(3, new String[]{"abc", "f'a", "fa\"b"}, Types.ARRAY);
+    pstmt.setObject(3, new String[]{stringWithNonAsciiWhiteSpace, "f'a", " \tfa\"b  "}, Types.ARRAY);
     pstmt.executeUpdate();
     pstmt.close();
 
@@ -118,11 +120,173 @@ public class ArrayTest extends BaseTest4 {
     String[] strarr = (String[]) arr.getArray(2, 2);
     assertEquals(2, strarr.length);
     assertEquals("f'a", strarr[0]);
-    assertEquals("fa\"b", strarr[1]);
+    assertEquals(" \tfa\"b  ", strarr[1]);
+
+    strarr = (String[]) arr.getArray();
+    assertEquals(stringWithNonAsciiWhiteSpace, strarr[0]);
 
     rs.close();
   }
 
+  @Test
+  public void testIndexAccess() throws SQLException {
+    final int[][][] origIntArray = new int[2][2][2];
+    final double[][][] origDblArray = new double[2][2][2];
+    final String[][][] origStringArray = new String[2][2][2];
+    final Object[][][] origIntObjArray = new Object[2][2][2];
+    final Object[][][] origDblObjArray = new Object[2][2][2];
+    final Object[][][] origStringObjArray = new Object[2][2][2];
+    int i = 0;
+    for (int x = 0; x < 2; ++x) {
+      for (int y = 0; y < 2; ++y) {
+        for (int z = 0; z < 2; ++z) {
+          origIntArray[x][y][z] = i;
+          origDblArray[x][y][z] = i / 10;
+          origStringArray[x][y][z] = Integer.toString(i);
+          origIntObjArray[x][y][z] = i;
+          origDblObjArray[x][y][z] = i / 10;
+          origStringObjArray[x][y][z] = Integer.toString(i);
+          i++;
+        }
+      }
+    }
+    PreparedStatement pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
+    pstmt.setObject(1, origIntArray[0][0], Types.ARRAY);
+    pstmt.setObject(2, origDblArray[0][0], Types.ARRAY);
+    pstmt.setObject(3, origStringArray[0][0], Types.ARRAY);
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    Statement stmt = conn.createStatement();
+    ResultSet rs = stmt.executeQuery("SELECT intarr[1], decarr[1], strarr[1] FROM arrtest");
+    Assert.assertTrue(rs.next());
+
+    assertEquals(origIntArray[0][0][0], rs.getInt(1));
+    assertEquals(origDblArray[0][0][0], rs.getDouble(2), 0.001);
+    assertEquals(origStringArray[0][0][0], rs.getString(3));
+    rs.close();
+    stmt.close();
+
+    pstmt = conn.prepareStatement("delete from arrtest");
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
+    pstmt.setObject(1, conn.createArrayOf("int4", origIntObjArray[0][0]), Types.ARRAY);
+    pstmt.setObject(2, conn.createArrayOf("float8", origDblObjArray[0][0]), Types.ARRAY);
+    pstmt.setObject(3, conn.createArrayOf("varchar",  origStringObjArray[0][0]), Types.ARRAY);
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    stmt = conn.createStatement();
+    rs = stmt.executeQuery("SELECT intarr[1], decarr[1], strarr[1] FROM arrtest");
+    Assert.assertTrue(rs.next());
+
+    assertEquals(origIntArray[0][0][0], rs.getInt(1));
+    assertEquals(origDblArray[0][0][0], rs.getDouble(2), 0.001);
+    assertEquals(origStringArray[0][0][0], rs.getString(3));
+    rs.close();
+    stmt.close();
+
+    pstmt = conn.prepareStatement("delete from arrtest");
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
+    pstmt.setObject(1, conn.createArrayOf("int4", origIntArray[0]), Types.ARRAY);
+    pstmt.setObject(2, conn.createArrayOf("float8", origDblArray[0]), Types.ARRAY);
+    pstmt.setObject(3, conn.createArrayOf("varchar",  origStringArray[0]), Types.ARRAY);
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    stmt = conn.createStatement();
+    rs = stmt.executeQuery("SELECT intarr[1][1], decarr[1][1], strarr[1][1], intarr[2][1], decarr[2][1], strarr[2][1] FROM arrtest");
+    Assert.assertTrue(rs.next());
+
+    assertEquals(origIntArray[0][0][0], rs.getInt(1));
+    assertEquals(origDblArray[0][0][0], rs.getDouble(2), 0.001);
+    assertEquals(origStringArray[0][0][0], rs.getString(3));
+    assertEquals(origIntArray[0][1][0], rs.getInt(4));
+    assertEquals(origDblArray[0][1][0], rs.getDouble(5), 0.001);
+    assertEquals(origStringArray[0][1][0], rs.getString(6));
+    rs.close();
+    stmt.close();
+
+    pstmt = conn.prepareStatement("delete from arrtest");
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
+    pstmt.setObject(1, conn.createArrayOf("int4", origIntObjArray[0]), Types.ARRAY);
+    pstmt.setObject(2, conn.createArrayOf("float8", origDblObjArray[0]), Types.ARRAY);
+    pstmt.setObject(3, conn.createArrayOf("varchar",  origStringObjArray[0]), Types.ARRAY);
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    stmt = conn.createStatement();
+    rs = stmt.executeQuery("SELECT intarr[1][1], decarr[1][1], strarr[1][1], intarr[2][1], decarr[2][1], strarr[2][1] FROM arrtest");
+    Assert.assertTrue(rs.next());
+
+    assertEquals(origIntArray[0][0][0], rs.getInt(1));
+    assertEquals(origDblArray[0][0][0], rs.getDouble(2), 0.001);
+    assertEquals(origStringArray[0][0][0], rs.getString(3));
+    assertEquals(origIntArray[0][1][0], rs.getInt(4));
+    assertEquals(origDblArray[0][1][0], rs.getDouble(5), 0.001);
+    assertEquals(origStringArray[0][1][0], rs.getString(6));
+    rs.close();
+    stmt.close();
+
+    pstmt = conn.prepareStatement("delete from arrtest");
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
+
+    pstmt.setObject(1, conn.createArrayOf("int4", origIntArray), Types.ARRAY);
+    pstmt.setObject(2, conn.createArrayOf("float8", origDblArray), Types.ARRAY);
+    pstmt.setObject(3, conn.createArrayOf("varchar",  origStringArray), Types.ARRAY);
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    stmt = conn.createStatement();
+    rs = stmt.executeQuery("SELECT intarr[1][1][1], decarr[1][1][1], strarr[1][1][1], intarr[2][1][1], decarr[2][1][1], strarr[2][1][1] FROM arrtest");
+    Assert.assertTrue(rs.next());
+
+    assertEquals(origIntArray[0][0][0], rs.getInt(1));
+    assertEquals(origDblArray[0][0][0], rs.getDouble(2), 0.001);
+    assertEquals(origStringArray[0][0][0], rs.getString(3));
+    assertEquals(origIntArray[1][0][0], rs.getInt(4));
+    assertEquals(origDblArray[1][0][0], rs.getDouble(5), 0.001);
+    assertEquals(origStringArray[1][0][0], rs.getString(6));
+    rs.close();
+    stmt.close();
+
+    pstmt = conn.prepareStatement("delete from arrtest");
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
+
+    pstmt.setObject(1, conn.createArrayOf("int4", origIntObjArray), Types.ARRAY);
+    pstmt.setObject(2, conn.createArrayOf("float8", origDblObjArray), Types.ARRAY);
+    pstmt.setObject(3, conn.createArrayOf("varchar",  origStringObjArray), Types.ARRAY);
+    pstmt.executeUpdate();
+    pstmt.close();
+
+    stmt = conn.createStatement();
+    rs = stmt.executeQuery("SELECT intarr[1][1][1], decarr[1][1][1], strarr[1][1][1], intarr[2][1][1], decarr[2][1][1], strarr[2][1][1] FROM arrtest");
+    Assert.assertTrue(rs.next());
+
+    assertEquals(origIntArray[0][0][0], rs.getInt(1));
+    assertEquals(origDblArray[0][0][0], rs.getDouble(2), 0.001);
+    assertEquals(origStringArray[0][0][0], rs.getString(3));
+    assertEquals(origIntArray[1][0][0], rs.getInt(4));
+    assertEquals(origDblArray[1][0][0], rs.getDouble(5), 0.001);
+    assertEquals(origStringArray[1][0][0], rs.getString(6));
+    rs.close();
+    stmt.close();
+  }
+
   @Test
   public void testSetPrimitiveArraysObjects() throws SQLException {
     PreparedStatement pstmt = conn.prepareStatement("INSERT INTO arrtest VALUES (?,?,?)");
@@ -207,7 +371,7 @@ public class ArrayTest extends BaseTest4 {
 
     // you need a lot of backslashes to get a double quote in.
     stmt.executeUpdate("INSERT INTO arrtest VALUES ('{1,2,3}','{3.1,1.4}', '"
-        + TestUtil.escapeString(conn, "{abc,f'a,\"fa\\\"b\",def}") + "')");
+        + TestUtil.escapeString(conn, "{abc,f'a,\"fa\\\"b\",def, un  quot\u000B \u2001 \r}") + "')");
 
     ResultSet rs = stmt.executeQuery("SELECT intarr, decarr, strarr FROM arrtest");
     Assert.assertTrue(rs.next());
@@ -234,6 +398,10 @@ public class ArrayTest extends BaseTest4 {
     Assert.assertEquals("f'a", strarr[0]);
     Assert.assertEquals("fa\"b", strarr[1]);
 
+    strarr = (String[]) arr.getArray();
+    assertEquals(5, strarr.length);
+    assertEquals("un  quot\u000B \u2001", strarr[4]);
+
     rs.close();
     stmt.close();
   }
@@ -242,9 +410,10 @@ public class ArrayTest extends BaseTest4 {
   public void testRetrieveResultSets() throws SQLException {
     Statement stmt = conn.createStatement();
 
+    final String stringWithNonAsciiWhiteSpace = "a\u2001b";
     // you need a lot of backslashes to get a double quote in.
     stmt.executeUpdate("INSERT INTO arrtest VALUES ('{1,2,3}','{3.1,1.4}', '"
-        + TestUtil.escapeString(conn, "{abc,f'a,\"fa\\\"b\",def}") + "')");
+        + TestUtil.escapeString(conn, "{\"a\u2001b\",f'a,\"fa\\\"b\",def}") + "')");
 
     ResultSet rs = stmt.executeQuery("SELECT intarr, decarr, strarr FROM arrtest");
     Assert.assertTrue(rs.next());
@@ -289,6 +458,13 @@ public class ArrayTest extends BaseTest4 {
     Assert.assertTrue(!arrrs.next());
     arrrs.close();
 
+    arrrs = arr.getResultSet(1, 1);
+    Assert.assertTrue(arrrs.next());
+    Assert.assertEquals(1, arrrs.getInt(1));
+    Assert.assertEquals(stringWithNonAsciiWhiteSpace, arrrs.getString(2));
+    Assert.assertFalse(arrrs.next());
+    arrrs.close();
+
     rs.close();
     stmt.close();
   }
@@ -391,6 +567,63 @@ public class ArrayTest extends BaseTest4 {
     Assert.assertNull(arr.toString());
   }
 
+  @Test
+  public void testDirectFieldString() throws SQLException {
+    Array arr = new PgArray((BaseConnection) conn, Oid.VARCHAR_ARRAY,
+        "{\" lead\t\",  unquot\u000B \u2001 \r, \" \fnew \n \"\t, \f\" \" }");
+    final String[] array = (String[]) arr.getArray();
+    assertEquals(4, array.length);
+    assertEquals(" lead\t", array[0]);
+    assertEquals(" \fnew \n ", array[2]);
+    assertEquals(" ", array[3]);
+
+    // PostgreSQL drops leading and trailing whitespace, so does the driver
+    assertEquals("unquot\u2001", array[1]);
+  }
+
+  @Test
+  public void testStringEscaping() throws SQLException {
+
+    final String stringArray = "{f'a,\"fa\\\"b\",def, un  quot\u000B \u2001 \r, someString }";
+
+    final Statement stmt = conn.createStatement();
+    try {
+
+      stmt.executeUpdate("INSERT INTO arrtest VALUES (NULL, NULL, '" + TestUtil.escapeString(conn, stringArray) + "')");
+
+      final ResultSet rs = stmt.executeQuery("SELECT strarr FROM arrtest");
+      Assert.assertTrue(rs.next());
+
+      Array arr = rs.getArray(1);
+      Assert.assertEquals(Types.VARCHAR, arr.getBaseType());
+      String[] strarr = (String[]) arr.getArray();
+      assertEquals(5, strarr.length);
+      assertEquals("f'a", strarr[0]);
+      assertEquals("fa\"b", strarr[1]);
+      assertEquals("def", strarr[2]);
+      assertEquals("un  quot\u000B \u2001", strarr[3]);
+      assertEquals("someString", strarr[4]);
+
+      rs.close();
+    } finally {
+      stmt.close();
+    }
+
+    final Array directArray = new PgArray((BaseConnection) conn, Oid.VARCHAR_ARRAY, stringArray);
+    final String[] actual = (String[]) directArray.getArray();
+    assertEquals(5, actual.length);
+    assertEquals("f'a", actual[0]);
+    assertEquals("fa\"b", actual[1]);
+    assertEquals("def", actual[2]);
+    assertEquals("someString", actual[4]);
+
+    // the driver strips out ascii white spaces from an unescaped string, even in
+    // the middle of the value. while this does not exactly match the behavior of
+    // the backend, it will always quote values where ascii white spaces are
+    // present, making this difference not worth the complexity involved addressing.
+    assertEquals("unquot\u2001", actual[3]);
+  }
+
   @Test
   public void testUnknownArrayType() throws SQLException {
     Statement stmt = conn.createStatement();
