@@ -90,8 +90,6 @@ public abstract class BaseDataSourceTest {
     bds.setUser(TestUtil.getUser());
     bds.setPassword(TestUtil.getPassword());
     bds.setPrepareThreshold(TestUtil.getPrepareThreshold());
-    bds.setLoggerLevel(TestUtil.getLogLevel());
-    bds.setLoggerFile(TestUtil.getLogFile());
     bds.setProtocolVersion(TestUtil.getProtocolVersion());
   }
 
