@@ -473,4 +473,25 @@ public class DatabaseMetaDataTest {
       lastType = type;
     }
   }
+
+  @Test
+  public void testGetSqlTypes() throws SQLException {
+    if (TestUtil.haveMinimumServerVersion(conn, ServerVersion.v10)) {
+      try (Connection privileged = TestUtil.openPrivilegedDB()) {
+        try (Statement stmt = privileged.createStatement()) {
+          // create a function called array_in
+          stmt.execute("CREATE OR REPLACE FUNCTION public.array_in(anyarray, oid, integer)\n"
+              + " RETURNS anyarray\n"
+              + " LANGUAGE internal\n"
+              + " STABLE PARALLEL SAFE STRICT\n"
+              + "AS $function$array_in$function$");
+        }
+        DatabaseMetaData dbmd = privileged.getMetaData();
+        ResultSet rs = dbmd.getTypeInfo();
+        try (Statement stmt = privileged.createStatement()) {
+          stmt.execute("drop function public.array_in(anyarray, oid, integer)");
+        }
+      }
+    }
+  }
 }
